import os

from .errors import *
from .decoder import PYONDecoder, is_pyon, is_json, convert, convert_json, convert_json_to_pyon
from .encoder import PYONEncoder

def dumps(obj):
    '''
    convert `obj` to string
    '''
    return PYONEncoder(obj).encode()

def loads(string):
    '''
    convert `string` (pyon) to python dict
    '''
    return PYONDecoder(string).decode()

class Read:
    def __init__(self, filepath:str, auto_save:bool=False):
        if not os.path.isfile(filepath):
            raise FileExistsError(filepath)
        if not isinstance(auto_save, bool):
            raise UnexpectedType(auto_save, 'Expected Bool')
        
        self.__filepath = filepath
        self.__auto_save = auto_save

        with open(self.__filepath, 'r') as file:
            self.__decoder = PYONDecoder(file.read())
            self.__file_data = self.__decoder.decode()

    def write(self, obj):
        '''
        write `obj` to file

        `obj` can be pyon string or json string or python dict
        '''
        filepath = self.__filepath
        obj_as_string = dumps(obj)
        
        # types handling
        if isinstance(obj, dict):
            with open(filepath, 'w') as file:
                file.write(obj_as_string)

            return True
        if is_pyon(obj_as_string):
            with open(filepath, 'w') as file:
                file.write(obj_as_string)

            return True
        if is_json(obj_as_string):
            with open(filepath, 'w') as file:
                file.write(convert_json_to_pyon(obj))

            return True

        
        with open(filepath, 'w') as file:
            file.write(obj_as_string)

    @property
    def readfile(self):
        '''
        reads file while still updating it (if a change was occure)
        '''
        with open(self.__filepath, 'r') as file:
            self.__decoder = PYONDecoder(file.read())
            self.__file_data = self.__decoder.decode()

        return self.__file_data

    @property
    def filepath(self):
        '''
        return filepath
        '''
        return self.__filepath

    @property
    def auto_save(self):
        '''
        returns if user wanted the file to be autosaved
        '''
        return self.__auto_save


from setuptools import setup, find_packages

# README.md
with open('README.md', 'r', encoding='utf-8') as readme_file:
    readme = readme_file.read()

setup(
    name='pyonr',
    packages=find_packages(),
    version='1.0.0b4',
    description='PYON Reader - Python Object Nation',
    author='Nawaf Alqari',
    author_email='nawafalqari13@gmail.com',
    keywords=['pyon', 'pyonr', 'json', 'pythonobjectnation', 'python object nation'],
    long_description=readme,
    long_description_content_type='text/markdown',
    project_urls={
        'Documentation': 'https://github.com/nawafalqari/pyon#readme',
        'Bug Tracker': 'https://github.com/nawafalqari/pyon/issues',
        'Source Code': 'https://github.com/nawafalqari/pyon/',
        'Discord': 'https://discord.gg/cpvynqk4XT'
    },
    license='MIT',
    url='https://github.com/nawafalqari/pyon/',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)


import ast
import json

from .errors import *

__all__ = ['PYONDecoder', 'is_json', 'is_pyon']

def is_json(obj_as_string:str):
    try:
        json.loads(obj_as_string)
    except ValueError:
        return False
    
    return True

def is_pyon(obj_as_string:str):
    try:
        ast.literal_eval(obj_as_string)
    except ValueError:
        return False

    return True

def convert(string:str):
    '''
    convert `string` to a `dict`
    works on json
    '''
    if not string:
        raise ArgumentTypeError(string, 'Was not expecting empty string')

    if is_json(string):
        return json.loads(string)
    if is_pyon(string):
        return ast.literal_eval(string)
        
    return None

def convert_json(obj_as_string):
    '''
    convert python dict to json string
    '''

    return json.dumps(obj_as_string)

def convert_json_to_pyon(obj_as_string):
    '''
    convert json string to pyon string
    '''
    obj_as_dict = json.loads(obj_as_string)
    obj_as_pyon = convert(obj_as_dict)

    return obj_as_pyon

class PYONDecoder:
    def __init__(self, obj_as_string:str):
        if not isinstance(obj_as_string, str):
            raise UnexpectedType(obj_as_string, f'Expected `str` not ({type(obj_as_string)})')

        self.obj = obj_as_string

    def decode(self):
        obj = self.obj
        if not obj:
            return None

        return convert(obj)
    
    def decode_json(self):
        '''
        convert obj to a json obj (string)
        '''
        obj = self.obj
        if not obj:
            return '{}'

        return 